﻿// 7.SWL      03.02.2015

using System;
using System.Windows;
using System.Collections;
using System.IO;
using System.Media;

// Morse Wave.

namespace CwWaveWpf
{
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        static string FileName = "cw.wav";     // Имя записываемого Wav файла.

        // Заголовок Wave файла.
        // Закомментированные переменные вычисляются в методе SaveWave.
        Int32 ChunkId = 0x46464952;     // ASCII текст "RIFF".
        // Int32 ChunkSize;              // Размер файла без строки "RIFF".
        Int32 Format = 0x45564157;      // ASCII текст "WAVE".
        Int32 SubChunk1Id = 0x20746D66; // ASCII текст "fmt " (включая пробел).
        Int32 SubChunk1Size = 16;       // Разрядность выборки амплитуды.
        Int16 AudioFormat = 1;          // Формат файла (для PCM = 1).
        Int16 NumChannels = 1;          // Моно = 1, стерео = 2;
        Int32 SampleRate = 8000;        // Частота дискретизации.
        // Int32 ByteRate;               // Скорость потока данных.
        Int16 BlockAlign = 2;           // Число байт в сэмпле.
        Int16 BitsPerSample = 16;       // Разрядность Дискретизации.
        Int32 SubChunk2Id = 0x61746164; // ASCII текст "data".
        //Int32 SubChunk2Size;            // Длина звуковых данных в байтах.
        Int16[] DataSound;              // Звуковые данные.

        // Параметры Cw сигнала.
        Int16 Amplitude = Int16.MaxValue;  // Максимальная положительная амплитуда сигнала (+32767).
        //Int16 Amplitude = Int16.MinValue;  // Максимальная отрицательная амплитуда сигнала (-32768).

        double Frequency = 800.00;         // Частота сигнала в герцах.        
        Int16 Point = 50; // Длительность точки в миллисекудах.
        Int16 Front = 8;  // Длительность фронта нарастания или спада амплитуды в миллисекудах.

        double Phase;   // Фаза сигнала.

        // Массив строк на передачу.
        string[] CwData = { "..-", "....", "--.", "--...", "...", ".--", ".-.." }; // uhg7swl.

        ArrayList DataList = new ArrayList(); // Инициализация коллекции.

        // Инициализация плеера.
        SoundPlayer cwPlay = new SoundPlayer(FileName);

        // КНОПКА PLAY CW.
        private void btnPlayCw_Click(object sender, RoutedEventArgs e)
        {
            Begin();
        }

        // КНОПКА STOP CW.
        private void btnStopCw_Click(object sender, RoutedEventArgs e)
        {
            cwPlay.Stop();     // Остановка проигрывания Wav файла.
        }

        //
        private void Begin()
        {
            MorseGenerator();    // Формирование Cw сигнала.

            SaveWaveFile();      // Запись Wav файла.            

            // Воспроизведение файла.
            cwPlay.Play();        // Один раз.

            //cwPlay.PlayLooping(); // Циклическое.
        }

        // ФОРМИРОВАНИЕ СИГНАЛА MORSE В ФОРМЕ ТРАПЕЦИИ.
        private void MorseGenerator()
        {
            // Проверка значения длительности фронта.
            if (Front == 0) Front = 1;

            DataList.Clear(); // Очистка коллекции.

            Phase = 0;        // Фаза сигнала.

            int duration;     // Длина посылки в циклах.

            Int16 point = (Int16)(SampleRate * Point / 1000);  // Длина точки байт.
            Int16 front = (Int16)(SampleRate * Front / 1000);  // Длина фронта байт.
            Int16 step = (Int16)(Amplitude / front);           // Шаг нарастания или спада амплитуды в байтах.

            // Коррекция максимального значения амплитуды  
            // до целого значения количества шагов нарастания или спада.
            Int16 ampl = (Int16)(Amplitude / step);
            Amplitude = (Int16)(ampl * step);

            // Передача массива кодов Морзе.
            for (int i = 0; i < CwData.Length; i++)
            {
                string code = CwData[i];    // Строка для передачи.

                for (int j = 0; j < code.Length; j++)
                {
                    if (code[j] == '.')
                    {
                        duration = point;     // Длительность точки.                   
                    }
                    else
                    {
                        duration = point * 3; // Длительность тире.
                    }

                    // Нарастание амплитуды переднего фронта.
                    for (int up = 0; up != Amplitude; up += step)
                    {
                        DataList.Add((Int16)(up * Math.Sin(Phase)));
                        Phase += 2 * Math.PI * Frequency / SampleRate;

                        if (Phase > 2 * Math.PI) Phase -= 2 * Math.PI;
                    }

                    // Посылка  с максимальной амплитудой.
                    for (int max = 0; max < duration - (front * 2); max++)
                    {
                        DataList.Add((Int16)(Amplitude * Math.Sin(Phase)));
                        Phase += 2 * Math.PI * Frequency / SampleRate;

                        if (Phase > 2 * Math.PI) Phase -= 2 * Math.PI;
                    }

                    // Спад амплитуды заднего фронта.
                    for (int down = Amplitude; down != 0; down -= step)
                    {
                        DataList.Add((Int16)(down * Math.Sin(Phase)));
                        Phase += 2 * Math.PI * Frequency / SampleRate;

                        if (Phase > 2 * Math.PI) Phase -= 2 * Math.PI;
                    }

                    // ==================== Пауза между точкой и тире. ======================================

                    for (int pause = 0; pause < point; pause++)
                    {
                        DataList.Add((Int16)0);
                    }
                }

                // ==================== Пауза между символами одного слова. ==============================

                for (int p = 0; p < point * 3; p++)
                {
                    DataList.Add((Int16)0);
                }
            }

            /* 
             * Замечание!!!
             * Дальше можно было бы использовать только коллекцию,
             * но для примера запишем данные в байтовый массив.
             */

            // Получение длины  данных  коллекции.
            Int32 sizeByte = DataList.Count;

            // Инициализация массива данных.
            DataSound = new Int16[sizeByte];

            // Копирование звуковых данных из коллекции в массив. 
            DataList.CopyTo(DataSound);
        }

        // ЗАПИСЬ WAVE ФАЙЛА.
        private void SaveWaveFile()
        {
            // Создание  файла и стыковка его с потоком.
            Stream newFile = File.Create(FileName);
            BinaryWriter file = new BinaryWriter(newFile);

            // ==================== ЗАГОЛОВОК (HEADER) WAVE ФАЙЛА. =======================
            file.Write(ChunkId);         // Символы в ASCII "RIFF".
            // ChunkSize - размер файла от данной точки.
            file.Write((Int32)(36 + DataSound.Length * BlockAlign));
            file.Write(Format);          // Символы в ASCII "WAVE".
            file.Write(SubChunk1Id);     // Символы в ASCII "frm " включая "пробел".
            file.Write(SubChunk1Size);   // Размер блока формата для PCM.
            file.Write(AudioFormat);     // 1 - PCM.
            file.Write(NumChannels);     // Количество каналов.
            file.Write(SampleRate);      // Частота дискретизации.
            // ByteRate - скорость потока данных.
            file.Write((Int32)(SampleRate * NumChannels * BitsPerSample / 8));
            file.Write(BlockAlign);      // Количество байт в блоке (сэмпле).
            file.Write(BitsPerSample);   // Точность звучания 8 - бит, 16 - бит и т.д..
            file.Write(SubChunk2Id);     // Символы в ASCII "data".
            // SubChunk2Size - размер звуковых данных в байтах.
            file.Write((Int32)(DataSound.Length * BlockAlign));
            // ===========================================================================

            // Записываем данные в поток.
            for (int i = 0; i < DataSound.Length; i++)
            {
                foreach (byte dataByte in BitConverter.GetBytes(DataSound[i]))
                {
                    newFile.WriteByte(dataByte);
                }
            }

            newFile.Close();   // Закрытие файла.        
        }
    }
}
