﻿// 7.SWL  03.02.2015
using System;
using System.Windows;
using System.IO;
using System.Media;
using System.Collections;

// Psk Wave.

namespace PskWaveWpf
{
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        static string FileName = "psk.wav";    // Имя записываемого Wav файла.

        // =================== ЗАГОЛОВОК (HEADER) WAVE ФАЙЛА =====================  
        // Закомментированные переменные вычисляются в методе SaveWave.
        Int32 ChunkId = 0x46464952;     // ASCII текст "RIFF".
        // Int32 ChunkSize;              // Размер файла без строки "RIFF".
        Int32 Format = 0x45564157;      // ASCII текст "WAVE".
        Int32 SubChunk1Id = 0x20746D66; // ASCII текст "fmt " (включая пробел).
        Int32 SubChunk1Size = 16;       // Разрядность выборки амплитуды.
        Int16 AudioFormat = 1;          // Формат файла (для PCM = 1).
        Int16 NumChannels = 1;          // Моно = 1, стерео = 2;
        Int32 SampleRate = 8000;        // Частота дискретизации.
        // Int32 ByteRate;               // Скорость потока данных.
        Int16 BlockAlign = 2;           // Число байт в сэмпле.
        Int16 BitsPerSample = 16;       // Разрядность Дискретизации.
        Int32 SubChunk2Id = 0x61746164; // ASCII текст "data".
        //Int32 SubChunk2Size;            // Длина звуковых данных в байтах.
        Int16[] DataSound;              // Звуковые данные.
        // =========================================================================

        // Параметры Psk сигнала.
        Int16 Amplitude = Int16.MaxValue;  // Максимальная положительная амплитуда сигнала (+32767).
        //Int16 Amplitude = Int16.MinValue;  // Максимальная отрицательная амплитуда сигнала (-32768).
        double Frequency = 1000.00;        // Частота сигнала в герцах.

        /* Baud       ms
         * 15.625    64
         * 20.0      50
         * 31.25     32
         * 62.5      16
         * 93.75     10.66
         * 125.0     8
         * 250.0     4
         */

        double Duration = 32.00;            // Длительность ms.

        double Phase;   // Фаза сигнала.

        // Массив варикодов.
        // Структура массива:               "mark(00)","varicode","mark(00)","varicode","..." и т.д.
        string[] VaricodeData = { "000000000", "00","101010111", "00", "101011", "00", "1011011", "00",  // Uhg
                                  "110101101", "00","10111","00", "1101011","00", "11010111","00",       // 7swL
                                  "00","11111","00","000000000"                                          // CR.
                                };

        ArrayList DataList = new ArrayList(); // Инициализация коллекции.

        SoundPlayer pskPlay = new SoundPlayer(FileName); // Инициализация плеера.

        // КНОПКА PLAY PSK.
        private void btnPlayPsk_Click(object sender, RoutedEventArgs e)
        {
            Begin();
        }

        // КНОПКА STOP PSK.
        private void btnStopPsk_Click(object sender, RoutedEventArgs e)
        {
            pskPlay.Stop();     // Остановка проигрывания Wav файла.
        }

        //
        private void Begin()
        {
            PskGenerator();  // Формирование Psk сигнала.

            SaveWaveFile();   // Запись Wav файла.

            // Вывод информации о сигнале.
            textBlockOut.Text = "  Frequency = " + Frequency + "Hz.  \n";

            // Math.Round округляет до указанного кол-во знаков после запятой (до двух).
            textBlockOut.Text += "  Baud = " + Math.Round(1000 / Duration, 2) + ".  \n";

            textBlockOut.Text += "  Duration = " + Math.Round(Duration, 2) + "ms.  ";


            // Воспроизведение.
            pskPlay.Play();        // Один раз.

            //pskPlay.PlayLooping(); // Циклически.
        }

        // ФОРМИРОВАНИЕ PSK СИГНАЛА/.
        private void PskGenerator()
        {  // Требует доработки как при формировании сигнала Морзе.

            DataList.Clear(); // Очистка коллекции.

            Phase = 0;        // Фаза сигнала.

            int cycle = (Int16)(SampleRate * Duration / 1000);  // Длина посылки байт.

            // Передача массива Psk кодов.
            for (int i = 0; i < VaricodeData.Length; i++)
            {
                string code = VaricodeData[i];    // Строка для передачи.

                for (int j = 0; j < code.Length; j++)
                {
                    if (code[j] == '0') Amplitude *= -1; // Инвертируем фазу.

                    for (int k = 0; k < cycle; k++)
                    {
                        DataList.Add((Int16)(Amplitude * Math.Cos(Phase)));
                        Phase += 2 * Math.PI * Frequency / SampleRate;

                        if (Phase > 2 * Math.PI) Phase -= 2 * Math.PI;
                    }
                }
            }

            /* 
             * Замечание!!!
             * Дальше можно было бы использовать только коллекцию,
             * но для примера запишем данные в байтовый массив.
             */

            // Получение длины  данных  коллекции.
            Int32 sizeByte = DataList.Count;

            // Инициализация массива данных.
            DataSound = new Int16[sizeByte];

            // Копирование звуковых данных из коллекции в массив. 
            DataList.CopyTo(DataSound);
        }

        // ЗАПИСЬ WAVE ФАЙЛА.
        private void SaveWaveFile()
        {
            // Создание  файла и стыковка его с потоком.
            Stream newFile = File.Create(FileName);
            BinaryWriter file = new BinaryWriter(newFile);

            // ==================== ЗАГОЛОВОК (HEADER) WAVE ФАЙЛА. =======================
            file.Write(ChunkId);         // Символы в ASCII "RIFF".
            // ChunkSize - размер файла от данной точки.
            file.Write((Int32)(36 + DataSound.Length * BlockAlign));
            file.Write(Format);          // Символы в ASCII "WAVE".
            file.Write(SubChunk1Id);     // Символы в ASCII "frm " включая "пробел".
            file.Write(SubChunk1Size);   // Размер блока формата для PCM.
            file.Write(AudioFormat);     // 1 - PCM.
            file.Write(NumChannels);     // Количество каналов.
            file.Write(SampleRate);      // Частота дискретизации.
            // ByteRate - скорость потока данных.
            file.Write((Int32)(SampleRate * NumChannels * BitsPerSample / 8));
            file.Write(BlockAlign);      // Количество байт в блоке (сэмпле).
            file.Write(BitsPerSample);   // Точность звучания 8 - бит, 16 - бит и т.д..
            file.Write(SubChunk2Id);     // Символы в ASCII "data".
            // SubChunk2Size - размер звуковых данных в байтах.
            file.Write((Int32)(DataSound.Length * BlockAlign));
            // ===========================================================================

            // Записываем данные в поток.
            for (int i = 0; i < DataSound.Length; i++)
            {
                foreach (byte dataByte in BitConverter.GetBytes(DataSound[i]))
                {
                    newFile.WriteByte(dataByte);
                }
            }

            newFile.Close();   // Закрытие файла.        
        }
    }
}
