﻿// 7.SWL   03.02.2015
using System;
using System.Windows;
using System.IO;
using System.Media;
using System.Collections;

// Rtty Wave.

namespace RttyWaveWpf
{
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        static string FileName = "rtty.wav";   // Имя записываемого Wav файла.

        // =================== ЗАГОЛОВОК (HEADER) WAVE ФАЙЛА =====================       
        // Закомментированные переменные вычисляются в методе SaveWave.
        Int32 ChunkId = 0x46464952;     // ASCII текст "RIFF".
        // Int32 ChunkSize;              // Размер файла без строки "RIFF".
        Int32 Format = 0x45564157;      // ASCII текст "WAVE".
        Int32 SubChunk1Id = 0x20746D66; // ASCII текст "fmt " (включая пробел).
        Int32 SubChunk1Size = 16;       // Разрядность выборки амплитуды.
        Int16 AudioFormat = 1;          // Формат файла (для PCM = 1).
        Int16 NumChannels = 1;          // Моно = 1, стерео = 2;
        Int32 SampleRate = 8000;        // Частота дискретизации.
        // Int32 ByteRate;               // Скорость потока данных.
        Int16 BlockAlign = 2;           // Число байт в сэмпле.
        Int16 BitsPerSample = 16;       // Разрядность Дискретизации.
        Int32 SubChunk2Id = 0x61746164; // ASCII текст "data".
        //Int32 SubChunk2Size;            // Длина звуковых данных в байтах.
        Int16[] DataSound;              // Звуковые данные.
        // =========================================================================

        // Параметры Rtty сигнала.
        Int16 Amplitude = Int16.MaxValue;  // Максимальная положительная амплитуда сигнала (+32767).
        //Int16 Amplitude = Int16.MinValue;  // Максимальная отрицательная амплитуда сигнала (-32768).
        double FrequencyBase = 1000.00;    // Основная частота сигнала в герцах (2125.00).

        // Популярные значения сдвига частот 170, 425 и 850 Гц.
        double FrequencyShift = 170;

        /* Стандартные значения скорости передачи.
         * 
          Baud    длина бита     слов/мин.  
          45.45     22 ms           60  
           50       20 ms           66  
           75      13.33 ms         100  
           100       10 ms          133 
         * 
         */
        double Duration = 22.00;           // Длительность посылки в миллисекудах.

        double Phase;   // Фаза сигнала.

        // Массив для передачи.
        // Порядок строк в массиве:       {"Stop/Start(110),"Rtty code","и т.д."};
        string[] RttyData = {  "1111111111","110","01000","110","00010","1111111","110","11111",   // Stop LF CR Лат регистр.  
                               "110","01010","110","10101","110","01010","110","10101","110","01010","110","10101", // RYRYRY
                               "110","00100","110","11100","110","00101","110","01011", // Пробел UHG
                               "110","11011","110","11100","110","11111",               // Циф регистр 7 Лат регистр 
                               "110","10100","110","11001","110","01001","110","00100", //  SWL пробел.
                               "110","01010","110","10101","110","01010","110","10101","110","01010","110","10101", // RYRYRY
                               "11111111111"};                  //  Stop

        // Инициализация коллекции.
        ArrayList DataList = new ArrayList();

        // Инициализация плеера.
        SoundPlayer rttyPlay = new SoundPlayer(FileName);

        // КНОПКА PLAY RTTY.
        private void btnPlayRtty_Click(object sender, RoutedEventArgs e)
        {           
            Begin();
        }

        // КНОПКА STOP RTTY.
        private void btnStopRtty_Click(object sender, RoutedEventArgs e)
        {
            rttyPlay.Stop();     // Остановка проигрывания Wav файла.
        }

        //
        private void Begin()
        {
            RttyGenerator();  // Формирование Rtty сигнала.

            SaveWaveFile();   // Запись Wav файла.

            // Вывод информации о сигнале.
            textBlockOut.Text = "  Frequency = " + FrequencyBase + "Hz.  \n";

            // Math.Round округляет до указанного кол-во знаков после запятой (до двух).
            textBlockOut.Text += "  Baud = " + Math.Round(1000 / Duration, 2) + ".  \n";
            textBlockOut.Text += "  Duration = " +  Duration + "ms.  ";
           
            // Воспроизведение.
            rttyPlay.Play();        // Один раз.

            //rttyPlay.PlayLooping(); // Циклически.
        }

        // ФОРМИРОВАНИЕ RTTY СИГНАЛА/.
        private void RttyGenerator()
        {
            DataList.Clear(); // Очистка коллекции.

            Phase = 0;        // Фаза сигнала.

            double frequency; // Частота нуля или единицы.

            int cycle = (Int16)(SampleRate * Duration / 1000);  // Длина посылки байт.

            // Передача массива Rtty кодов.
            for (int i = 0; i < RttyData.Length; i++)
            {
                string code = RttyData[i];    // Строка для передачи.

                for (int j = 0; j < code.Length; j++)
                {
                    if (code[j] == '0')
                    {
                        frequency = FrequencyBase;     // Частота нуля.                   
                    }
                    else
                    {   //             Rtty-LSB(-), Rtty-USB(+).
                        frequency = FrequencyBase + FrequencyShift; // (+-) Частота единицы. 
                    }

                    for (int k = 0; k < cycle; k++)
                    {
                        DataList.Add((Int16)(Amplitude * Math.Sin(Phase)));
                        Phase += 2 * Math.PI * frequency / SampleRate;

                        if (Phase > 2 * Math.PI) Phase -= 2 * Math.PI;
                    }
                }
            }

            /* 
             * Замечание!!!
             * Дальше можно было бы использовать только коллекцию,
             * но для примера запишем данные в байтовый массив.
             */

            // Получение длины  данных  коллекции.
            Int32 sizeByte = DataList.Count;

            // Инициализация массива данных.
            DataSound = new Int16[sizeByte];

            // Копирование звуковых данных из коллекции в массив. 
            DataList.CopyTo(DataSound);
        }

        // ЗАПИСЬ WAVE ФАЙЛА.
        private void SaveWaveFile()
        {
            // Создание  файла и стыковка его с потоком.
            Stream newFile = File.Create(FileName);
            BinaryWriter file = new BinaryWriter(newFile);

            // ==================== ЗАГОЛОВОК (HEADER) WAVE ФАЙЛА. =======================
            file.Write(ChunkId);         // Символы в ASCII "RIFF".
            // ChunkSize - размер файла от данной точки.
            file.Write((Int32)(36 + DataSound.Length * BlockAlign));
            file.Write(Format);          // Символы в ASCII "WAVE".
            file.Write(SubChunk1Id);     // Символы в ASCII "frm " включая "пробел".
            file.Write(SubChunk1Size);   // Размер блока формата для PCM.
            file.Write(AudioFormat);     // 1 - PCM.
            file.Write(NumChannels);     // Количество каналов.
            file.Write(SampleRate);      // Частота дискретизации.
            // ByteRate - скорость потока данных.
            file.Write((Int32)(SampleRate * NumChannels * BitsPerSample / 8));
            file.Write(BlockAlign);      // Количество байт в блоке (сэмпле).
            file.Write(BitsPerSample);   // Точность звучания 8 - бит, 16 - бит и т.д..
            file.Write(SubChunk2Id);     // Символы в ASCII "data".
            // SubChunk2Size - размер звуковых данных в байтах.
            file.Write((Int32)(DataSound.Length * BlockAlign));
            // ===========================================================================

            // Записываем данные в поток.
            for (int i = 0; i < DataSound.Length; i++)
            {
                foreach (byte dataByte in BitConverter.GetBytes(DataSound[i]))
                {
                    newFile.WriteByte(dataByte);
                }
            }

            newFile.Close();   // Закрытие файла.        
        }
    }
}
