﻿// 7.SWL   03.02.2015
using System;
using System.Windows;
using System.Collections;
using System.IO;
using System.Media;

// Wave - It's easy!!!    File.

namespace SaveWaveFile
{
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        static string FileName = "wave.wav";   // Имя записываемого Wav файла.

        // Заголовок Wave файла.
        // Закомментированные переменные вычисляются в методе SaveWave.
        Int32 ChunkId = 0x46464952;     // ASCII текст "RIFF".
        // Int32 ChunkSize;              // Размер файла без строки "RIFF".
        Int32 Format = 0x45564157;      // ASCII текст "WAVE".
        Int32 SubChunk1Id = 0x20746D66; // ASCII текст "fmt " (включая пробел).
        Int32 SubChunk1Size = 16;       // Разрядность выборки амплитуды.
        Int16 AudioFormat = 1;          // Формат файла (для PCM = 1).
        Int16 NumChannels = 1;          // Моно = 1, стерео = 2;
        Int32 SampleRate = 44100;       // Частота дискретизации.
        // Int32 ByteRate;               // Скорость потока данных.
        Int16 BlockAlign = 2;           // Число байт в сэмпле.
        Int16 BitsPerSample = 16;       // Разрядность Дискретизации.
        Int32 SubChunk2Id = 0x61746164; // ASCII текст "data".
        //Int32 SubChunk2Size;            // Длина звуковых данных в байтах.
        Int16[] DataSound;              // Звуковые данные.

        // Параметры сигнала.
        Int16 Amplitude = Int16.MaxValue;  // Максимальная положительная амплитуда сигнала (+32767).
        //Int16 Amplitude = Int16.MinValue;  // Максимальная отрицательная амплитуда сигнала (-32768).

        double Frequency = 800.00;         // Частота сигнала в герцах.        
        double Seconds = 1.50;             // Продолжительность сигнала в секудах.
        double Phase;                      // Фаза сигнала.

        ArrayList DataList = new ArrayList(); // Инициализация коллекции.
        SoundPlayer wavePlay = new SoundPlayer(FileName); // Инициализация плеера.

        // КНОПКА PLAY WAVE.
        private void btnPlayWave_Click(object sender, RoutedEventArgs e)
        {
            Begin();
        }

        // КНОПКА STOP WAVE.
        private void btnStopWave_Click(object sender, RoutedEventArgs e)
        {
            wavePlay.Stop(); // Остановка проигрывания Wav файла.
        }

        //
        private void Begin()
        {
            Generator();    // Формирование сигнала.

            SaveWaveFile(); // Запись Wav файла.

            // Вывод информации о сигнале.
            textBlockOut.Text = "  Frequency = " + Frequency + "Hz.  \n";
            textBlockOut.Text += "  Period = " + Math.Round(1000 / Frequency, 3) + " ms.  \n";
            textBlockOut.Text += "  Duration = " + Seconds + " Sec.  ";


            //Воспроизведение.
            wavePlay.Play();    // Один раз.

            //wavePlay.PlayLooping(); // Циклически.
        }

        // ФОРМИРОВАНИЕ СИГНАЛА.
        private void Generator()
        {
            DataList.Clear(); // Очистка коллекции.

            Phase = 0;        // Фаза сигнала.

            for (int i = 0; i < SampleRate * Seconds; i++)
            {
                // Синусоидальный сигнал.
                DataList.Add((Int16)(Amplitude * Math.Sin(Phase)));

                // Косинусоидальный сигнал.
                //DataList.Add((Int16)(Amplitude * Math.Cos(Phase)));

                // Пауза (тишина).
                //DataList.Add((Int16)(0);

                Phase += (2 * Math.PI * Frequency) / SampleRate;

                if (Phase > 2 * Math.PI)
                {
                    Phase -= 2 * Math.PI;
                }
            }

            /* 
             * Замечание!!!
             * Дальше можно было бы использовать только коллекцию,
             * но для примера запишем данные в байтовый массив.
             */

            // Получение длины  данных  коллекции.
            Int32 sizeByte = DataList.Count;

            // Инициализация массива данных.
            DataSound = new Int16[sizeByte];

            // Копирование звуковых данных в массив. 
            DataList.CopyTo(DataSound);
        }

        // ЗАПИСЬ WAVE ФАЙЛА.
        private void SaveWaveFile()
        {
            // Создание  файла и стыковка его с потоком.
            Stream newFile = File.Create(FileName);
            BinaryWriter file = new BinaryWriter(newFile);

            // ==================== ЗАГОЛОВОК (HEADER) WAVE ФАЙЛА. =======================
            file.Write(ChunkId);         // Символы в ASCII "RIFF".
            // ChunkSize - размер файла от данной точки.
            file.Write((Int32)(36 + DataSound.Length * BlockAlign));
            file.Write(Format);          // Символы в ASCII "WAVE".
            file.Write(SubChunk1Id);     // Символы в ASCII "frm " включая "пробел".
            file.Write(SubChunk1Size);   // Размер блока формата для PCM.
            file.Write(AudioFormat);     // 1 - PCM.
            file.Write(NumChannels);     // Количество каналов.
            file.Write(SampleRate);      // Частота дискретизации.
            // ByteRate - скорость потока данных.
            file.Write((Int32)(SampleRate * NumChannels * BitsPerSample / 8));
            file.Write(BlockAlign);      // Количество байт в блоке (сэмпле).
            file.Write(BitsPerSample);   // Точность звучания 8 - бит, 16 - бит и т.д..
            file.Write(SubChunk2Id);     // Символы в ASCII "data".
            // SubChunk2Size - размер звуковых данных в байтах.
            file.Write((Int32)(DataSound.Length * BlockAlign));
            // ===========================================================================

            // Записываем данные в поток.
            for (int i = 0; i < DataSound.Length; i++)
            {
                foreach (byte dataByte in BitConverter.GetBytes(DataSound[i]))
                {
                    newFile.WriteByte(dataByte);
                }
            }

            newFile.Close();   // Закрытие файла.        
        }
    }
}
