﻿// 7.SWL 03.02.2015

using System;
using System.Windows;
using System.Threading;
using System.Media;
using System.Runtime.InteropServices;

// Simple programming code Morse.

namespace SimpleCwWpf
{
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        /* Параметры сигнала Морзе.
         * Тире = точка * 3.
         * Пауза межту посылками внутри знака = точка.
         * Пауза между знаками = точка * 3 или тире.
         * Пауза между словами (группами) = точка * 7.
         */

        int Frequency = 1000; // Частота сигнала для Console Beep и kernel32 в герцах.
        int Point = 100;      // Длительность точки в миллисекундах.

        // Массив строк кодов Морзе для Console и kernel32.
        string[] CwMessage = { "--...", "...", ".--", ".-.." }; // 7SWL;

        // КНОПКА BEEP.
        private void btnBeepConsoleString_Click(object sender, RoutedEventArgs e)
        {
            CwBeep();
        }

        // BEEP - ПЕРЕДАЧА СООБЩЕНИЯ.
        private void CwBeep()
        {
            int durMessage = CwMessage.Length; // Кол-во знаков в сообщении;

            for (int i = 0; i < durMessage; i++)
            {
                string code = CwMessage[i];    // Знак для передачи.

                for (int j = 0; j < code.Length; j++)
                {
                    // Передача точки.
                    if (code[j] == '.')
                    {
                        Console.Beep(Frequency, Point);
                    }

                        // Передача тире.
                    else
                    {
                        Console.Beep(Frequency, Point * 3);
                    }

                    Thread.Sleep(Point);   // Пауза между посылками внутри знака.
                }

                Thread.Sleep(Point * 3);   // Пауза между знаками.
            }
        }

        // КНОПКА WAVE FILE.
        private void btnWaveFile_Click(object sender, RoutedEventArgs e)
        {
            WaveFile();
        }

        // FILE - ПЕРЕДАЧА СООБЩЕНИЯ.
        private void WaveFile()
        {
            string cwFile = "800.wav"; // Звуковой файл.

            SoundPlayer cw = new SoundPlayer(cwFile);

            int durMessage = CwMessage.Length; // Кол-во знаков в сообщении;

            for (int i = 0; i < durMessage; i++)
            {
                string code = CwMessage[i];    // Знак для передачи.

                for (int j = 0; j < code.Length; j++)
                {
                    // Передача точки.
                    if (code[j] == '.')
                    {
                        cw.Play();
                        Thread.Sleep(Point); // Длительность точки.
                        cw.Stop();
                    }

                     // Передача тире.
                    else
                    {
                        cw.Play();
                        Thread.Sleep(Point * 3); // Длительность тире.
                        cw.Stop();
                    }

                    Thread.Sleep(Point);   // Пауза между посылками внутри знака.
                }

                Thread.Sleep(Point * 3);   // Пауза между знаками.
            }
        }

        // КНОПКА BEEP CONSOLE BYTE ARRAY.
        private void btnBeepConsoleByteArray_Click(object sender, RoutedEventArgs e)
        {
            CwByteArray();
        }

        // ПЕРЕДАЧА CW СИГНАЛА BYTE.
        private void CwByteArray()
        {
            // Массив байтов на передачу.

            // *Первый вариант.
            //                     код, цикл, маска
            byte[] byteMessage = { 0x18, 5, 0x10, //* 7
                                   0x00, 3, 0x04, //* S
                                   0x03, 3, 0x04, //* W
                                   0x04, 4, 0x08  //* L
                                 };

            // **Второй вариант.
            //                       код, цикл
            //byte[] byteMessage = { 0x18, 5, //** 7
            //                       0x00, 3, //** S
            //                       0x03, 3, //** W
            //                       0x04, 4  //** L
            //                     }; 

            int durMessage = byteMessage.Length; // Длина массива;

            for (int i = 0; i < durMessage; i++)
            {
                // *Первый вариант.
                byte code = byteMessage[i++];    //* Код для передачи.
                byte cycle = byteMessage[i++];   //* Цикл.
                byte mask = byteMessage[i];      //* Маска.


                // **Второй вариант. 
                //byte code = byteMessage[i++];  //** Код для передачи.
                //byte cycle = byteMessage[i];   //** Цикл.
                //byte mask = 1;                 //** Маска - 0000 0001;
                //mask <<= cycle - 1;            //** Сдвиг бита маски влево.


                for (int j = 0; j < cycle; j++)
                {
                    if ((code & mask) == 0)  // Проверка бита.
                    {// 0 - точка.
                        Console.Beep(Frequency, Point);
                    }
                    else
                    {// 1 - тире.
                        Console.Beep(Frequency, Point * 3);
                    }

                    Thread.Sleep(Point);   // Пауза между посылками внутри знака.

                    mask >>= 1;            // Следующий бит (сдвиг бита маски вправо).
                }

                Thread.Sleep(Point * 3);   // Пауза между знаками.
            }
        }

        // КНОПКА KERNEL32.DLL.
        private void btnKernel32_Click(object sender, RoutedEventArgs e)
        {
            BeepKernel32();
        }

        // Подключение библиотеки kernel32.dll.
        [DllImport("kernel32.dll", EntryPoint = "Beep", SetLastError = true, ExactSpelling = true)]
        public static extern bool Beep(int frequency, int duration);

        // Генерация звука.
        private void BeepKernel32()
        {
            int durMessage = CwMessage.Length; // Кол-во знаков в сообщении;

            for (int i = 0; i < durMessage; i++)
            {
                string code = CwMessage[i];    // Знак для передачи.

                for (int j = 0; j < code.Length; j++)
                {
                    // Передача точки.
                    if (code[j] == '.')
                    {
                        Beep(Frequency, Point);
                    }

                        // Передача тире.
                    else
                    {
                        Beep(Frequency, Point * 3);
                    }

                    Thread.Sleep(Point);   // Пауза между посылками внутри знака.
                }

                Thread.Sleep(Point * 3);   // Пауза между знаками.
            }
        }
    }
}
